@@ -37,7 +37,7 @@ class DAVPrincipal extends Principal
   private $calendar_free_busy_set;
 
   /**
-  * @var RFC3744: The principals that are direct members of this group.
+  * @var RFC3744: Whether this is a group principal.
   */
   protected $_is_group;
 
@@ -232,10 +232,10 @@ class DAVPrincipal extends Principal
         }
       }
     }
-//      @dbg_error_log( 'principal', 'Read-proxy-for:    %s', implode(',',$this->read_proxy_for) );
-//      @dbg_error_log( 'principal', 'Write-proxy-for:   %s', implode(',',$this->write_proxy_for) );
-//      @dbg_error_log( 'principal', 'Read-proxy-group:  %s', implode(',',$this->read_proxy_group) );
-//      @dbg_error_log( 'principal', 'Write-proxy-group: %s', implode(',',$this->write_proxy_group) );
+      dbg_error_log( 'principal', 'Read-proxy-for:    %s', implode(',',$this->read_proxy_for) );
+      dbg_error_log( 'principal', 'Write-proxy-for:   %s', implode(',',$this->write_proxy_for) );
+      dbg_error_log( 'principal', 'Read-proxy-group:  %s', implode(',',$this->read_proxy_group) );
+      dbg_error_log( 'principal', 'Write-proxy-group: %s', implode(',',$this->write_proxy_group) );
   }
 
 
@@ -313,7 +313,7 @@ class DAVPrincipal extends Principal
     if ( isset($this->{$property_id}) ) {
       if ( ! is_object($this->{$property_id}) ) return $this->{$property_id};
       return clone($this->{$property_id});
-    } 
+    }
     return null;
   }
 
@@ -376,30 +376,27 @@ class DAVPrincipal extends Principal
 
 
   /**
-	 * The property calendar-free-busy-set has been dropped from draft 5 of the scheduling extensions for CalDAV,
-	 * and is not present in the final official RFC (6638).
-	 * Hence, by default we do not call this method and do not populate the property.
-	 * Enable the config value $c->support_obsolete_free_busy_property if you really need it, however be aware that
-	 * performance may be adversely affected if you do so, since the resulting query over the collection table is slow.
+  * The property calendar-free-busy-set has been dropped from draft 5 of the scheduling extensions for CalDAV,
+  * and is not present in the final official RFC (6638).
+  * Hence, by default we do not call this method and do not populate the property.
+  * Enable the config value $c->support_obsolete_free_busy_property if you really need it, however be aware that
+  * performance may be adversely affected if you do so, since the resulting query over the collection table is slow.
   */
   function calendar_free_busy_set() {
-		if (!isset($this->calendar_free_busy_set))
-		{
-			$this->calendar_free_busy_set = array();
-			$qry = new AwlQuery('SELECT dav_name FROM collection WHERE is_calendar AND (schedule_transp = \'opaque\' OR schedule_transp IS NULL) AND dav_name ~ :dav_name_start ORDER BY user_no, collection_id',
-					array(':dav_name_start' => '^' . $this->dav_name));
-			if ($qry->Exec('principal', __LINE__, __FILE__))
-			{
-				while ($calendar = $qry->Fetch())
-				{
-					$this->calendar_free_busy_set[] = ConstructURL($calendar->dav_name, true);
-				}
-			}
-		}
+    if (!isset($this->calendar_free_busy_set)) {
+      $this->calendar_free_busy_set = array();
+      $qry = new AwlQuery('SELECT dav_name FROM collection WHERE is_calendar AND (schedule_transp = \'opaque\' OR schedule_transp IS NULL) AND dav_name ~ :dav_name_start ORDER BY user_no, collection_id',
+                      array(':dav_name_start' => '^' . $this->dav_name));
+      if ($qry->Exec('principal', __LINE__, __FILE__)) {
+        while ($calendar = $qry->Fetch()) {
+          $this->calendar_free_busy_set[] = ConstructURL($calendar->dav_name, true);
+        }
+      }
+    }
     return $this->calendar_free_busy_set;
   }
 
-  
+
   /**
   * Return the privileges bits for the current session user to this resource
   */
@@ -410,7 +407,7 @@ class DAVPrincipal extends Principal
     if ( $this->_is_group ) {
       if ( isset($session->principal) && in_array($session->principal->url(), $this->GroupMemberSet()) ) {
         $this->privileges |=  privilege_to_bits( array('DAV::read', 'DAV::read-current-user-privilege-set') );
-      } 
+      }
     }
     return $this->privileges;
   }
@@ -451,9 +448,9 @@ class DAVPrincipal extends Principal
   * Returns properties which are specific to this principal
   */
   function PrincipalProperty( $tag, $prop, &$reply, &$denied ) {
-		global $c;
+    global $c, $request;
 
-    dbg_error_log('principal',': RenderAsXML: Principal Property "%s"', $tag );
+    dbg_error_log('principal',':PrincipalProperty: Principal Property "%s"', $tag );
     switch( $tag ) {
       case 'DAV::getcontenttype':
         $reply->DAVElement( $prop, 'getcontenttype', 'httpd/unix-directory' );
@@ -488,8 +485,18 @@ class DAVPrincipal extends Principal
 
       case 'http://calendarserver.org/ns/:group-member-set':
       case 'DAV::group-member-set':
-        if ( ! $this->_is_group ) return false;
-        $reply->DAVElement( $prop, 'group-member-set', $reply->href($this->group_member_set) );
+        if ( $request->IsProxyRequest() ) {
+          /** calendar-proxy-{read,write} pseudo-principal, see caldav-proxy 3.2 */
+          if ($request->proxy_type == 'read') {
+            $reply->DAVElement( $prop, 'group-member-set', $reply->href($this->ReadProxyGroup()) );
+          } else {
+            $reply->DAVElement( $prop, 'group-member-set', $reply->href($this->WriteProxyGroup()) );
+          }
+        } else {
+          /** regular group principal */
+          if ( ! $this->_is_group ) return false;
+          $reply->DAVElement( $prop, 'group-member-set', $reply->href($this->group_member_set) );
+        }
         break;
 
       case 'http://calendarserver.org/ns/:group-membership':
@@ -532,19 +539,20 @@ class DAVPrincipal extends Principal
         break;
 
       case 'urn:ietf:params:xml:ns:caldav:calendar-free-busy-set':
-				/** Note that this property was dropped from the scheduling extensions for CalDAV.
-				 * We only populate it if the config value support_obsolete_free_busy_property is set.
-				 * This should not be enabled unless your CalDAV client requires the property; beware
-				 * that doing so may also adversely affect PROPFIND performance. */
-				if ( isset($c->support_obsolete_free_busy_property) && $c->support_obsolete_free_busy_property )
-	        $reply->CalDAVElement( $prop, 'calendar-free-busy-set', $reply->href( $this->calendar_free_busy_set() ) );
-				else
-					return false;
-				break;
+        /** Note that this property was dropped from the scheduling extensions for CalDAV.
+        * We only populate it if the config value support_obsolete_free_busy_property is set.
+        * This should not be enabled unless your CalDAV client requires the property; beware
+        * that doing so may also adversely affect PROPFIND performance.
+        */
+        if ( isset($c->support_obsolete_free_busy_property) && $c->support_obsolete_free_busy_property )
+          $reply->CalDAVElement( $prop, 'calendar-free-busy-set', $reply->href( $this->calendar_free_busy_set() ) );
+        else
+          return false;
+        break;
 
       case 'urn:ietf:params:xml:ns:caldav:calendar-user-address-set':
-				$reply->CalDAVElement($prop, 'calendar-user-address-set', $reply->href($this->user_address_set));
-				break;
+        $reply->CalDAVElement($prop, 'calendar-user-address-set', $reply->href($this->user_address_set));
+        break;
 
       case 'DAV::owner':
         // After a careful reading of RFC3744 we see that this must be the principal-URL of the owner
@@ -579,9 +587,7 @@ class DAVPrincipal extends Principal
   * @return string An XML fragment with the requested properties for this principal
   */
   function RenderAsXML( $properties, &$reply, $props_only = false ) {
-    global $request;
-
-    dbg_error_log('principal',': RenderAsXML: Principal "%s"', $this->username );
+    dbg_error_log('principal',':RenderAsXML: Principal "%s"', $this->username );
 
     $prop = new XMLElement('prop');
     $denied = array();
