@@ -405,7 +405,7 @@ EOSQL;
       $this->collection->dav_name = $this->dav_name;
       $this->collection->dav_displayname = sprintf( '%s proxy %s', $matches[2], $matches[3] );
       $this->collection->exists = true;
-      $this->collection->parent_container = $matches[1] . '/';
+      $this->collection->parent_container = '/' . $matches[2] . '/';
     }
     else if ( preg_match( '#^(/[^/]+)/?$#', $this->dav_name, $matches)
            || preg_match( '#^((/principals/[^/]+/)[^/]+)/?$#', $this->dav_name, $matches) ) {
@@ -427,8 +427,8 @@ EOSQL;
 SELECT collection.*, path_privs(:session_principal::int8, collection.dav_name,:scan_depth::int), p.principal_id,
     p.type_id AS principal_type_id, p.displayname AS principal_displayname, p.default_privileges AS principal_default_privileges,
     timezones.vtimezone, dav_binding.access_ticket_id, dav_binding.parent_container AS bind_parent_container,
-		dav_binding.dav_displayname, owner.dav_name AS bind_owner_url, dav_binding.dav_name AS bound_to,  
-    dav_binding.external_url AS external_url, dav_binding.type AS external_type, dav_binding.bind_id AS bind_id 
+    dav_binding.dav_displayname, owner.dav_name AS bind_owner_url, dav_binding.dav_name AS bound_to,
+    dav_binding.external_url AS external_url, dav_binding.type AS external_type, dav_binding.bind_id AS bind_id
 FROM dav_binding
     LEFT JOIN collection ON (collection.collection_id=bound_source_id)
     LEFT JOIN principal p USING (user_no)
@@ -481,15 +481,15 @@ EOSQL;
         $this->collection->dav_name = preg_replace('{/[^/]*$}', '/', $this->dav_name);
       }
     }
-    
+
   }
-  
+
   /**
   * Find the collection associated with this resource.
   */
   protected function FetchCollection() {
     global $session;
-    
+
     /**
     * RFC4918, 8.3: Identifiers for collections SHOULD end in '/'
     *    - also discussed at more length in 5.2
@@ -528,7 +528,7 @@ EOSQL;
       }
       @dbg_error_log( 'DAVResource', ':FetchCollection: Read cached collection named "%s" of type "%s".', $this->collection->dav_name, $this->collection->type );
     }
-    
+
     if ( isset($this->collection->bound_from) ) {
       $this->_is_binding = true;
       $this->bound_from = str_replace( $this->collection->bound_to, $this->collection->bound_from, $this->dav_name);
@@ -537,7 +537,7 @@ EOSQL;
         $this->tickets[] = new DAVTicket($this->collection->access_ticket_id);
       }
     }
-    
+
     $this->_is_collection = ( $this->_is_principal || $this->collection->dav_name == $this->dav_name || $this->collection->dav_name == $this->dav_name.'/' );
     if ( $this->_is_collection ) {
       $this->dav_name = $this->collection->dav_name;
@@ -598,8 +598,6 @@ EOSQL;
   * Retrieve the actual resource.
   */
   protected function FetchResource() {
-    global $c, $session;
-
     if ( isset($this->exists) ) return;   // True or false, we've got what we can already
     if ( $this->_is_collection ) return;   // We have all we're going to read
 
@@ -667,7 +665,7 @@ EOQRY;
       dbg_error_log( 'ERROR', "Error occurred in:\n%s\n",$xml);
       if ($errno >= 200 && $errno < 300 && count($xml_tags) >= 3) {
           // XML namespace error, but parsing was probably fine: continue and return tags (cf. #9)
-          dbg_error_log( 'ERROR', 'XML namspace error but tags extracted, trying to continue');
+          dbg_error_log( 'ERROR', 'XML namespace error but tags extracted, trying to continue');
       } else {
           return $raw_string;
       }
@@ -754,9 +752,10 @@ EOQRY;
     return $this->parent;
   }
 
-  
+
   /**
   * Fetch the parent to this resource. This is deprecated - use GetParentContainer() instead.
+  * @deprecated
   */
   function FetchParentContainer() {
     deprecated('DAVResource::FetchParentContainer');
@@ -1312,7 +1311,7 @@ EOQRY;
     dbg_error_log('DAVResource', 'Request for a%scached sync-token', ($cachedOK ? ' ' : 'n un') );
     if ( $this->IsPrincipal() ) return null;
     if ( $this->collection_id() == 0 ) return null;
-    if ( !isset($this->sync_token) || !$cachedOK ) { 
+    if ( !isset($this->sync_token) || !$cachedOK ) {
       $sql = 'SELECT new_sync_token( 0, :collection_id) AS sync_token';
       $params = array( ':collection_id' => $this->collection_id());
       $qry = new AwlQuery($sql, $params );
@@ -1325,7 +1324,7 @@ EOQRY;
     dbg_error_log('DAVResource', 'Returning sync token of "%s"', $this->sync_token );
     return $this->sync_token;
   }
-  
+
   /**
   * Checks whether the target collection is publicly_readable
   */
@@ -1401,8 +1400,6 @@ EOQRY;
   * Return ACL settings
   */
   function GetACL( &$xmldoc ) {
-    global $c, $session;
-
     if ( !isset($this->principal) ) $this->FetchPrincipal();
     $default_privs = $this->principal->default_privileges;
     if ( isset($this->collection->default_privileges) ) $default_privs = $this->collection->default_privileges;
@@ -1433,8 +1430,6 @@ EOQRY;
   * Return general server-related properties, in plain form
   */
   function GetProperty( $name ) {
-    global $c, $session;
-
 //    dbg_error_log( 'DAVResource', ':GetProperty: Fetching "%s".', $name );
     $value = null;
 
@@ -1473,7 +1468,7 @@ EOQRY;
         dbg_error_log( 'DAVResource', ':GetProperty: dav-data: fetched resource does%s exist.', ($this->exists?'':' not') );
         return $this->resource->caldav_data;
         break;
-        
+
       case 'principal':
         if ( !isset($this->principal) ) $this->FetchPrincipal();
         return clone($this->principal);
@@ -1483,7 +1478,7 @@ EOQRY;
         if ( isset($this->{$name}) ) {
           if ( ! is_object($this->{$name}) ) return $this->{$name};
           return clone($this->{$name});
-        } 
+        }
         if ( $this->_is_principal ) {
           if ( !isset($this->principal) ) $this->FetchPrincipal();
           if ( isset($this->principal->{$name}) ) return $this->principal->{$name};
@@ -1503,7 +1498,7 @@ EOQRY;
         if ( isset($this->{$name}) ) {
           if ( ! is_object($this->{$name}) ) return $this->{$name};
           return clone($this->{$name});
-        } 
+        }
         // dbg_error_log( 'DAVResource', ':GetProperty: Failed to find property "%s" on "%s".', $name, $this->dav_name );
     }
 
@@ -1537,7 +1532,7 @@ EOQRY;
     global $c, $session, $request;
 
 //    dbg_error_log( 'DAVResource', 'Processing "%s" on "%s".', $tag, $this->dav_name );
-    
+
     if ( $reply === null ) $reply = $GLOBALS['reply'];
 
     switch( $tag ) {
@@ -1694,7 +1689,7 @@ EOQRY;
         if ( isset($c->disable_caldav_proxy) && $c->disable_caldav_proxy ) return false;
         if ( !isset($proxy_type) ) $proxy_type = 'write';
         // ProxyFor is an already constructed URL
-		$this->FetchPrincipal();
+        $this->FetchPrincipal();
         $reply->CalendarserverElement($prop, 'calendar-proxy-'.$proxy_type.'-for', $reply->href( $this->principal->ProxyFor($proxy_type) ) );
         break;
 
@@ -1910,8 +1905,6 @@ EOQRY;
   * @return string An XML fragment with the requested properties for this principal
   */
   function RenderAsXML( $properties, &$reply, $bound_parent_path = null ) {
-    global $session, $c, $request;
-
     dbg_error_log('DAVResource',':RenderAsXML: Resource "%s" exists(%d)', $this->dav_name, $this->Exists() );
 
     if ( !$this->Exists() ) return null;
