@@ -21,7 +21,7 @@ require_once('vCalendar.php');
 require_once('WritableCollection.php');
 require_once('schedule-functions.php');
 include_once('iSchedule.php');
-include_once('RRule-v2.php');
+include_once('RRule.php');
 
 $bad_events = null;
 
@@ -98,7 +98,7 @@ function controlRequestContainer( $username, $user_no, $path, $caldav_context, $
     $sql = 'SELECT * FROM collection WHERE dav_name = :dav_name';
     $qry = new AwlQuery( $sql, array( ':dav_name' => $request_container) );
     if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) {
-      rollback_on_error( $caldav_context, $user_no, $path );
+      rollback_on_error( $caldav_context, $user_no, $path, 'Database error in: '.$sql );
     }
     if ( !isset($c->readonly_webdav_collections) || $c->readonly_webdav_collections == true ) {
       if ( $qry->rows() == 0 ) {
@@ -127,12 +127,12 @@ VALUES( :user_no, :parent_container, :dav_name, :dav_etag, :dav_displayname, TRU
     }
     else if ( isset($public) ) {
       $collection = $qry->Fetch();
-      if ( empty($collection->is_public) ) $collection->is_public = 'f'; 
+      if ( empty($collection->is_public) ) $collection->is_public = 'f';
       if ( $collection->is_public == ($public?'t':'f')  ) {
         $sql = 'UPDATE collection SET publicly_readable = :is_public::boolean WHERE collection_id = :collection_id';
         $params = array( ':is_public' => ($public?'t':'f'), ':collection_id' => $collection->collection_id );
         if ( ! $qry->QDo($sql,$params) ) {
-          rollback_on_error( $caldav_context, $user_no, $path );
+          rollback_on_error( $caldav_context, $user_no, $path, 'Database error in: '.$sql );
         }
       }
     }
@@ -166,6 +166,7 @@ function public_events_only( $user_no, $dav_name ) {
   return false;
 }
 
+
 /**
  * Get a TZID string from this VEVENT/VTODO/... component if we can
  * @param vComponent $comp
@@ -180,6 +181,7 @@ function GetTZID( vComponent $comp ) {
   return $p->GetParameterValue('TZID');
 }
 
+
 /**
 * Deliver scheduling requests to attendees
 * @param vComponent $ical the VCALENDAR to deliver
@@ -215,7 +217,7 @@ function handle_schedule_request( $ical ) {
 
     if ( isset($c->enable_auto_schedule) && !$c->enable_auto_schedule ) {
       // In this case we're being asked not to do auto-scheduling, so we build
-      // a response back for the client saying we can't...  
+      // a response back for the client saying we can't...
       $attendee->SetParameterValue ('SCHEDULE-STATUS','5.3;No scheduling support for user');
       continue;
     }
@@ -268,6 +270,7 @@ function handle_schedule_request( $ical ) {
   $request->DoResponse( 201, 'Created' );
 }
 
+
 /**
 * Deliver scheduling replies to organizer and other attendees
 * @param vComponent $ical the VCALENDAR to deliver
@@ -369,11 +372,11 @@ function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
   $schedule_reply = GetItip(new vCalendar($schedule_original->Render(null, true)), 'REPLY', $attendee->Value(), array('CUTYPE'=>true, 'SCHEDULE-STATUS'=>true));
 
   dbg_error_log( 'PUT', 'Writing scheduling REPLY from %s to %s', $request->principal->email(), $organizer_principal->email() );
-  
+
   $response = '3.7'; // Organizer was not found on server.
   if ( !$organizer_calendar->Exists() ) {
     dbg_error_log('ERROR','Default calendar at "%s" does not exist for user "%s"',
-                              $organizer_calendar->dav_name(), $schedule_target->username());
+                              $organizer_calendar->dav_name(), $organizer_principal->username());
     $response = '5.2'; // No scheduling support for user
   }
   else {
@@ -384,7 +387,7 @@ function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
       $response = '1.2'; // Scheduling reply delivered successfully
       if ( $organizer_calendar->WriteCalendarMember($schedule_original, false, false, $segment_name) === false ) {
         dbg_error_log('ERROR','Could not write updated calendar member to %s',
-                $attendee_calendar->dav_name(), $attendee_calendar->dav_name(), $schedule_target->username());
+                $organizer_calendar->dav_name());
         trace_bug('Failed to write scheduling resource.');
       }
     }
@@ -397,7 +400,7 @@ function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
   $organizer->SetParameterValue( 'SCHEDULE-STATUS', $response );
   $resource->UpdateOrganizerStatus($organizer);
   $scheduling_actions = true;
-  
+
   $calling_attendee = clone($attendee);
   $attendees = $schedule_original->GetAttendees();
   foreach( $attendees AS $attendee ) {
@@ -440,9 +443,9 @@ function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
       $r = new DAVResource($row);
       $attendee_calendar = new WritableCollection(array('path' => $r->parent_path()));
       if ($attendee_calendar->IsCalendar()) {
-        dbg_error_log( 'XXX', "found the event in attendee's calendar %s", $attendee_calendar->dav_name() );
+        dbg_error_log( 'PUT', "found the event in attendee's calendar %s", $attendee_calendar->dav_name() );
       } else {
-        dbg_error_log( 'XXX', 'could not find the event in any calendar, using schedule-default-calendar');
+        dbg_error_log( 'PUT', 'could not find the event in any calendar, using schedule-default-calendar');
         $attendee_calendar = new WritableCollection(array('path' => $schedule_target->internal_url('schedule-default-calendar')));
       }
       if ( !$attendee_calendar->Exists() ) {
@@ -458,7 +461,7 @@ function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
         else if ( $attendee_inbox->WriteCalendarMember($schedule_request, false) !== false ) {
           $response = '1.2'; // Scheduling invitation delivered successfully
           if ( $attendee_calendar->WriteCalendarMember($schedule_original, false) === false ) {
-            dbg_error_log('ERROR','Could not write updated calendar member to %s', 
+            dbg_error_log('ERROR','Could not write updated calendar member to %s',
                     $attendee_calendar->dav_name(), $attendee_calendar->dav_name(), $schedule_target->username());
             trace_bug('Failed to write scheduling resource.');
           }
@@ -468,11 +471,11 @@ function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
     dbg_error_log( 'PUT', 'Status for attendee <%s> set to "%s"', $attendee->Value(), $response );
     $attendee->SetParameterValue( 'SCHEDULE-STATUS', $response );
     $scheduling_actions = true;
-    
+
     $resource->UpdateAttendeeStatus($email, clone($attendee));
-    
+
   }
-  
+
   return $scheduling_actions;
 }
 
@@ -487,7 +490,7 @@ function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
 function do_scheduling_requests( vCalendar $resource, $create, $old_data = null ) {
   global $request, $c;
   if ( !isset($request) || (isset($c->enable_auto_schedule) && !$c->enable_auto_schedule) ) return false;
-  
+
   if ( ! is_object($resource) ) {
     trace_bug( 'do_scheduling_requests called with non-object parameter (%s)', gettype($resource) );
     return  false;
@@ -578,9 +581,9 @@ function do_scheduling_requests( vCalendar $resource, $create, $old_data = null
         $r = new DAVResource($row);
         $attendee_calendar = new WritableCollection(array('path' => $r->parent_path()));
         if ($attendee_calendar->IsCalendar()) {
-          dbg_error_log( 'XXX', "found the event in attendee's calendar %s", $attendee_calendar->dav_name() );
+          dbg_error_log( 'PUT', "found the event in attendee's calendar %s", $attendee_calendar->dav_name() );
         } else {
-          dbg_error_log( 'XXX', 'could not find the event in any calendar, using schedule-default-calendar');
+          dbg_error_log( 'PUT', 'could not find the event in any calendar, using schedule-default-calendar');
           $attendee_calendar = new WritableCollection(array('path' => $schedule_target->internal_url('schedule-default-calendar')));
         }
       }
@@ -653,7 +656,7 @@ function do_scheduling_requests( vCalendar $resource, $create, $old_data = null
 */
 function import_collection( $import_content, $user_no, $path, $caldav_context, $appending = false ) {
   global $c;
-  
+
   if ( ! ini_get('open_basedir') && (isset($c->dbg['ALL']) || isset($c->dbg['put'])) ) {
     $fh = fopen('/var/log/davical/PUT-2.debug','w');
     if ( $fh ) {
@@ -663,11 +666,11 @@ function import_collection( $import_content, $user_no, $path, $caldav_context, $
   }
 
   if ( preg_match( '{^begin:(vcard|vcalendar)}i', $import_content, $matches) ) {
-    if ( $matches[1] == 'VCARD' )
+    if ( strtoupper($matches[1]) == 'VCARD' )
       import_addressbook_collection( $import_content, $user_no, $path, $caldav_context, $appending );
-    elseif ( $matches[1] == 'VCALENDAR' )
+    elseif ( strtoupper($matches[1]) == 'VCALENDAR' )
       import_calendar_collection( $import_content, $user_no, $path, $caldav_context, $appending );
-  
+
     // Uncache anything to do with the collection
     $cache = getCacheInstance();
     $cache_ns = 'collection-'.preg_replace( '{/[^/]*$}', '/', $path);
@@ -675,9 +678,10 @@ function import_collection( $import_content, $user_no, $path, $caldav_context, $
   }
   else {
     dbg_error_log('PUT', 'Can only import files which are VCARD or VCALENDAR');
-  } 
+  }
 }
 
+
 /**
 * This function will import a whole addressbook
 * @param string $vcard_content the vcf file to import
@@ -694,34 +698,47 @@ function import_addressbook_collection( $vcard_content, $user_no, $path, $caldav
 
   $sql = 'SELECT * FROM collection WHERE dav_name = :dav_name';
   $qry = new AwlQuery( $sql, array( ':dav_name' => $path) );
-  if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) rollback_on_error( $caldav_context, $user_no, $path );
+  if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) rollback_on_error( $caldav_context, $user_no, $path, 'Database error in: '.$sql );
   if ( ! $qry->rows() == 1 ) {
     dbg_error_log( 'ERROR', ' PUT: Collection does not exist at "%s" for user %d', $path, $user_no );
-    rollback_on_error( $caldav_context, $user_no, $path );
+    rollback_on_error( $caldav_context, $user_no, $path, sprintf('Error: Collection does not exist at "%s" for user %d', $path, $user_no ));
   }
   $collection = $qry->Fetch();
-  
+  $collection_id = $collection->collection_id;
+
+  // Fetch the current collection data
+  $qry->QDo('SELECT dav_name, caldav_data FROM caldav_data WHERE collection_id=:collection_id', array(
+          ':collection_id' => $collection_id
+      ));
+  $current_data = array();
+  while( $row = $qry->Fetch() )
+    $current_data[$row->dav_name] = $row->caldav_data;
+
   if ( !(isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import) ) $qry->Begin();
   $base_params = array(
-       ':collection_id' => $collection->collection_id,
+       ':collection_id' => $collection_id,
        ':session_user' => $session->user_no,
        ':caldav_type' => 'VCARD'
   );
-  if ( !$appending ) {
-    if ( !$qry->QDo('DELETE FROM caldav_data WHERE collection_id = :collection_id', $base_params) )
-    rollback_on_error( $caldav_context, $user_no, $collection->collection_id );
-  }
-  
+
   $dav_data_insert = <<<EOSQL
 INSERT INTO caldav_data ( user_no, dav_name, dav_etag, caldav_data, caldav_type, logged_user, created, modified, collection_id )
     VALUES( :user_no, :dav_name, :etag, :dav_data, :caldav_type, :session_user, :created, :modified, :collection_id )
 EOSQL;
-  
+
+  $dav_data_update = <<<EOSQL
+UPDATE caldav_data SET user_no=:user_no, caldav_data=:dav_data, dav_etag=:etag, caldav_type=:caldav_type, logged_user=:session_user,
+  modified=current_timestamp WHERE collection_id=:collection_id AND dav_name=:dav_name
+EOSQL;
+
 
   $resources = $addressbook->GetComponents();
+  if ( count($resources) > 0 )
+    $qry->QDo('SELECT new_sync_token(0,'.$collection_id.')');
+
   foreach( $resources AS $k => $resource ) {
     if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Begin();
-  
+
     $vcard = new vCard( $resource->Render() );
 
     $uid = $vcard->GetPValue('UID');
@@ -743,36 +760,69 @@ EOSQL;
     }
 
     $rendered_card = $vcard->Render();
-    
+
+    // We don't allow any of &?\/@%+: in the UID to appear in the path, but anything else is fair game.
+    $dav_name = sprintf( '%s%s.vcf', $path, preg_replace('{[&?\\/@%+:]}','',$uid) );
+
     $dav_data_params = $base_params;
     $dav_data_params[':user_no'] = $user_no;
-    // We don't allow any of &?\/@%+: in the UID to appear in the path, but anything else is fair game.
-    $dav_data_params[':dav_name'] = sprintf( '%s%s.vcf', $path, preg_replace('{[&?\\/@%+:]}','',$uid) );
+    $dav_data_params[':dav_name'] = $dav_name;
     $dav_data_params[':etag'] = md5($rendered_card);
     $dav_data_params[':dav_data'] = $rendered_card;
     $dav_data_params[':modified'] = $last_modified;
     $dav_data_params[':created'] = $created;
-        
+
+    // Do we actually need to do anything?
+    $inserting = true;
+    if ( isset($current_data[$dav_name]) ) {
+      if ( $rendered_card == $current_data[$dav_name] ) {
+        unset($current_data[$dav_name]);
+        continue;
+      }
+      $sync_change = 200;
+      unset($current_data[$dav_name]);
+      $inserting = false;
+    }
+    else
+      $sync_change = 201;
+
     if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Begin();
-    
-    if ( !$qry->QDo($dav_data_insert,$dav_data_params) ) rollback_on_error( $caldav_context, $user_no, $path );
-  
-    $qry->QDo('SELECT dav_id FROM caldav_data WHERE dav_name = :dav_name ', array(':dav_name' => $dav_data_params[':dav_name']));
+
+    // Write to the caldav_data table
+    if ( !$qry->QDo( ($inserting ? $dav_data_insert : $dav_data_update), $dav_data_params) )
+      rollback_on_error( $caldav_context, $user_no, $path, 'Database error on:'. ($inserting ? $dav_data_insert : $dav_data_update));
+
+    // Get the dav_id for this row
+    $qry->QDo('SELECT dav_id FROM caldav_data WHERE dav_name = :dav_name ', array(':dav_name' => $dav_name));
     if ( $qry->rows() == 1 && $row = $qry->Fetch() ) {
       $dav_id = $row->dav_id;
     }
-      
-    $vcard->Write( $row->dav_id, false );
-    
+
+    $vcard->Write( $row->dav_id, !$inserting );
+
+    $qry->QDo("SELECT write_sync_change( $collection_id, $sync_change, :dav_name)", array(':dav_name' => $dav_name ) );
+
     if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Commit();
   }
-  
-  if ( !(isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import) ) {
-    if ( ! $qry->Commit() ) rollback_on_error( $caldav_context, $user_no, $path);
+
+  if ( !$appending && count($current_data) > 0 ) {
+    $params = array( ':collection_id' => $collection_id );
+    if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Begin();
+    foreach( $current_data AS $dav_name => $data ) {
+      $params[':dav_name'] = $dav_name;
+      $qry->QDo('DELETE FROM caldav_data WHERE collection_id = :collection_id AND dav_name = :dav_name', $params);
+      $qry->QDo('SELECT write_sync_change(:collection_id, 404, :dav_name)', $params);
+    }
+    if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Commit();
   }
-  
+
+  if ( !(isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import) ) {
+    if ( ! $qry->Commit() ) rollback_on_error( $caldav_context, $user_no, $path, 'Database error on COMMIT');
+  }
+
 }
 
+
 /**
 * This function will import a whole calendar
 * @param string $ics_content the ics file to import
@@ -803,14 +853,14 @@ function import_calendar_collection( $ics_content, $user_no, $path, $caldav_cont
       $after = $after->epoch();
     }
   }
-  
+
   $displayname = $calendar->GetPValue('X-WR-CALNAME');
   if ( !$appending && isset($displayname) ) {
     $sql = 'UPDATE collection SET dav_displayname = :displayname WHERE dav_name = :dav_name';
     $qry = new AwlQuery( $sql, array( ':displayname' => $displayname, ':dav_name' => $path) );
-    if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) rollback_on_error( $caldav_context, $user_no, $path );
+    if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) rollback_on_error( $caldav_context, $user_no, $path, 'Database error on: '.$sql );
   }
-  
+
 
   $tz_ids    = array();
   foreach( $timezones AS $k => $tz ) {
@@ -839,14 +889,14 @@ function import_calendar_collection( $ics_content, $user_no, $path, $caldav_cont
 
   $sql = 'SELECT * FROM collection WHERE dav_name = :dav_name';
   $qry = new AwlQuery( $sql, array( ':dav_name' => $path) );
-  if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) rollback_on_error( $caldav_context, $user_no, $path );
+  if ( ! $qry->Exec('PUT',__LINE__,__FILE__) ) rollback_on_error( $caldav_context, $user_no, $path, 'Database error on: '.$sql );
   if ( ! $qry->rows() == 1 ) {
     dbg_error_log( 'ERROR', ' PUT: Collection does not exist at "%s" for user %d', $path, $user_no );
-    rollback_on_error( $caldav_context, $user_no, $path );
+    rollback_on_error( $caldav_context, $user_no, $path, sprintf( 'Error: Collection does not exist at "%s" for user %d', $path, $user_no ));
   }
   $collection = $qry->Fetch();
   $collection_id = $collection->collection_id;
-  
+
   // Fetch the current collection data
   $qry->QDo('SELECT dav_name, caldav_data FROM caldav_data WHERE collection_id=:collection_id', array(
           ':collection_id' => $collection_id
@@ -854,7 +904,7 @@ function import_calendar_collection( $ics_content, $user_no, $path, $caldav_cont
   $current_data = array();
   while( $row = $qry->Fetch() )
     $current_data[$row->dav_name] = $row->caldav_data;
-  
+
   if ( !(isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import) ) $qry->Begin();
   $base_params = array( ':collection_id' => $collection_id );
 
@@ -867,7 +917,7 @@ EOSQL;
 UPDATE caldav_data SET user_no=:user_no, caldav_data=:dav_data, dav_etag=:etag, caldav_type=:caldav_type, logged_user=:session_user,
   modified=current_timestamp WHERE collection_id=:collection_id AND dav_name=:dav_name
 EOSQL;
-  
+
   $calitem_insert = <<<EOSQL
 INSERT INTO calendar_item (user_no, dav_name, dav_id, dav_etag, uid, dtstamp, dtstart, dtend, summary, location, class, transp,
                     description, rrule, tz_id, last_modified, url, priority, created, due, percent_complete, status, collection_id )
@@ -883,11 +933,11 @@ UPDATE calendar_item SET user_no=:user_no, dav_etag=:etag, uid=:uid, dtstamp=:dt
                 due=:due, percent_complete=:percent_complete, status=:status
        WHERE collection_id=:collection_id AND dav_name=:dav_name
 EOSQL;
-  
+
   $last_olson = '';
   if ( count($resources) > 0 )
     $qry->QDo('SELECT new_sync_token(0,'.$collection_id.')');
-  
+
   foreach( $resources AS $uid => $resource ) {
 
     /** Construct the VCALENDAR data */
@@ -895,9 +945,9 @@ EOSQL;
     $vcal->SetComponents($resource);
     $icalendar = $vcal->Render();
     $dav_name = sprintf( '%s%s.ics', $path, preg_replace('{[&?\\/@%+:]}','',$uid) );
- 
+
     if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Begin();
-    
+
     /** As ever, we mostly deal with the first resource component */
     $first = $resource[0];
 
@@ -944,7 +994,7 @@ EOSQL;
 
     // Write to the caldav_data table
     if ( !$qry->QDo( ($inserting ? $dav_data_insert : $dav_data_update), $dav_data_params) )
-      rollback_on_error( $caldav_context, $user_no, $path );
+      rollback_on_error( $caldav_context, $user_no, $path, 'Database error on:'. ($inserting ? $dav_data_insert : $dav_data_update));
 
     // Get the dav_id for this row
     $qry->QDo('SELECT dav_id FROM caldav_data WHERE dav_name = :dav_name ', array(':dav_name' => $dav_data_params[':dav_name']));
@@ -984,7 +1034,7 @@ EOSQL;
         if ( empty($dtstart_prop) ) {
           dbg_error_log('PUT','Invalid VEVENT without DTSTART, UID="%s" in collection %d', $uid, $collection_id);
           continue;
-        } 
+        }
         $value_type = $dtstart_prop->GetParameterValue('VALUE');
         dbg_error_log('PUT','DTSTART without DTEND. DTSTART value type is %s', $value_type );
         if ( isset($value_type) && $value_type == 'DATE' )
@@ -1061,7 +1111,7 @@ EOSQL;
     write_attendees($dav_id, $vcal);
 
     $qry->QDo("SELECT write_sync_change( $collection_id, $sync_change, :dav_name)", array(':dav_name' => $dav_name ) );
-    
+
     do_scheduling_requests( $vcal, true );
     if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Commit();
   }
@@ -1076,7 +1126,7 @@ EOSQL;
     }
     if ( isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import ) $qry->Commit();
   }
-  
+
   if ( !(isset($c->skip_bad_event_on_import) && $c->skip_bad_event_on_import) ) {
     if ( ! $qry->Commit() ) rollback_on_error( $caldav_context, $user_no, $path);
   }
@@ -1188,7 +1238,7 @@ function write_attendees( $dav_id, vCalendar $ical ) {
 /**
 * Actually write the resource to the database.  All checking of whether this is reasonable
 * should be done before this is called.
-* 
+*
 * @param DAVResource $resource The resource being written
 * @param string $caldav_data The actual data to be written
 * @param DAVResource $collection The collection containing the resource being written
@@ -1199,7 +1249,7 @@ function write_attendees( $dav_id, vCalendar $ical ) {
 * @param string Either 'INSERT' or 'UPDATE': the type of action we are doing
 * @param boolean $log_action Whether to log the fact that we are writing this into an action log (if configured)
 * @param string $weak_etag An etag that is NOT modified on ATTENDEE changes for this event
-* 
+*
 * @return boolean True for success, false for failure.
 */
 function write_resource( DAVResource $resource, $caldav_data, DAVResource $collection, $author, &$etag, $put_action_type, $caldav_context, $log_action=true, $weak_etag=null ) {
@@ -1223,7 +1273,7 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
     }
     $resource_type = $first->GetType();
   }
-  
+
   $collection_id = $collection->collection_id();
 
   $qry = new AwlQuery();
@@ -1249,7 +1299,7 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   }
   if ( $qry->rows() != 1 || !($row = $qry->Fetch()) ) {
     // No dav_id?  => We're toast!
-    trace_bug( 'No dav_id for "%s" on %s!!!', $path, ($create_resource ? 'create': 'update'));
+    trace_bug( 'No dav_id for "%s" on %s!!!', $path, ($put_action_type == 'INSERT' ? 'create': 'update'));
     rollback_on_error( $caldav_context, $user_no, $path);
     return false;
   }
@@ -1259,14 +1309,14 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   $calitem_params[':dav_id'] = $dav_id;
 
   $due = null;
-  if ( $first->GetType() == 'VTODO' ) $due = $first->GetPValue('DUE'); 
+  if ( $first->GetType() == 'VTODO' ) $due = $first->GetPValue('DUE');
   $calitem_params[':due'] = $due;
   $dtstart = $first->GetPValue('DTSTART');
   if ( empty($dtstart) ) $dtstart = $due;
   if (preg_match("/^1[0-8][0-9][0-9][01][0-9][0-3][0-9]$/", $dtstart))
      $dtstart = $dtstart . "T000000Z";
   $calitem_params[':dtstart'] = $dtstart;
-  
+
   $dtend = $first->GetPValue('DTEND');
   if ( isset($dtend) && $dtend != '' ) {
     dbg_error_log( 'PUT', ' DTEND: "%s", DTSTART: "%s", DURATION: "%s"', $dtend, $dtstart, $first->GetPValue('DURATION') );
@@ -1322,7 +1372,7 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
 
   $created = $first->GetPValue('CREATED');
   if ( $created == '00001231T000000Z' ) $created = '20001231T000000Z';
-  
+
   $class = $first->GetPValue('CLASS');
   /* Check and see if we should over ride the class. */
   /** @todo is there some way we can move this out of this function? Or at least get rid of the need for the SQL query here. */
@@ -1348,7 +1398,7 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
     foreach( $timezones AS $k => $tz ) {
       if ( $tz->GetPValue('TZID') != $tzid ) {
         /**
-        * We'll skip any tz definitions that are for a TZID other than the DTSTART/DUE on the first VEVENT/VTODO 
+        * We'll skip any tz definitions that are for a TZID other than the DTSTART/DUE on the first VEVENT/VTODO
         */
         dbg_error_log( 'ERROR', ' Event uses TZID[%s], skipping included TZID[%s]!', $tz->GetPValue('TZID'), $tzid );
         continue;
@@ -1425,7 +1475,7 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
     return false;
   }
 
-  
+
   if ( $put_action_type == 'INSERT' ) {
     $sql = <<<EOSQL
 INSERT INTO calendar_item (user_no, dav_name, dav_id, dav_etag, uid, dtstamp,
@@ -1465,7 +1515,7 @@ EOSQL;
     dbg_error_log( 'PUT', 'No log_caldav_action( %s, %s, %s, %s, %s) can be called.',
             $put_action_type, $first->GetPValue('UID'), $user_no, $collection_id, $path );
   }
-  
+
   $qry = new AwlQuery( $sql, $calitem_params );
   if ( !$qry->Exec('PUT',__LINE__,__FILE__) ) {
     rollback_on_error( $caldav_context, $user_no, $path);
@@ -1477,12 +1527,12 @@ EOSQL;
   if ( function_exists('post_commit_action') ) {
     post_commit_action( $put_action_type, $first->GetPValue('UID'), $user_no, $collection_id, $path );
   }
-  
+
   // Uncache anything to do with the collection
   $cache = getCacheInstance();
   $cache_ns = 'collection-'.preg_replace( '{/[^/]*$}', '/', $path);
   $cache->delete( $cache_ns, null );
-  
+
   dbg_error_log( 'PUT', 'User: %d, ETag: %s, Path: %s', $author, $etag, $path);
 
   return true;  // Success!
