@@ -48,19 +48,19 @@ function handle_subaction( $subaction ) {
   global $delete_ticket_confirmation_required;
   global $delete_bind_in_confirmation_required;
   global $delete_binding_confirmation_required;
-  
+  global $can_write_principal;
+
   dbg_error_log('admin-principal-edit',':handle_action: Action %s', $subaction );
 
   switch( $subaction ) {
     case 'delete_collection':
       dbg_error_log('admin-principal-edit',':handle_action: Deleting collection %s for principal %d', $_GET['dav_name'], $id );
-      if ( $session->AllowedTo('Admin')
-                || ($id > 0 && $session->principal_id == $id) ) {
+      if ($can_write_principal) {
         if ( $session->CheckConfirmationHash('GET', 'confirm') ) {
           dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete collection %s for principal %d', $_GET['dav_name'], $id );
           $qry = new AwlQuery('DELETE FROM collection WHERE dav_name=?;', $_GET['dav_name'] );
           if ( $qry->Exec() ) {
-            $c->messages[] = i18n('Collection deleted');
+            $c->messages[] = i18n('Collection deleted.');
             return true;
           }
           else {
@@ -74,6 +74,9 @@ function handle_subaction( $subaction ) {
           return false;
         }
       }
+      else {
+        $c->messages[] = i18n('You are not allowed to delete collections for this principal.');
+      }
       break;
 
     case 'delete_principal':
@@ -83,7 +86,7 @@ function handle_subaction( $subaction ) {
           dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete principal %d -%s', $id );
           $qry = new AwlQuery('DELETE FROM dav_principal WHERE principal_id=?', $id );
           if ( $qry->Exec() ) {
-            $c->messages[] = i18n('Principal deleted');
+            $c->messages[] = i18n('Principal deleted.');
             return true;
           }
           else {
@@ -97,17 +100,19 @@ function handle_subaction( $subaction ) {
           return false;
         }
       }
+      else {
+        $c->messages[] = i18n('You are not allowed to delete principals.');
+      }
       break;
 
     case 'delete_ticket':
       dbg_error_log('admin-principal-edit',':handle_action: Deleting ticket "%s" for principal %d', $_GET['ticket_id'], $id );
-      if ( $session->AllowedTo('Admin')
-                || ($id > 0 && $session->principal_id == $id) ) {
+      if ($can_write_principal) {
         if ( $session->CheckConfirmationHash('GET', 'confirm') ) {
           dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete ticket "%s" for principal %d', $_GET['ticket_id'], $id );
           $qry = new AwlQuery('DELETE FROM access_ticket WHERE ticket_id=?;', $_GET['ticket_id'] );
           if ( $qry->Exec() ) {
-            $c->messages[] = i18n('Access ticket deleted');
+            $c->messages[] = i18n('Access ticket deleted.');
             return true;
           }
           else {
@@ -121,18 +126,20 @@ function handle_subaction( $subaction ) {
           return false;
         }
       }
+      else {
+        $c->messages[] = i18n('You are not allowed to delete tickets for this principal.');
+      }
       break;
 
     case 'delete_bind_in':
     case 'delete_binding':
       dbg_error_log('admin-principal-edit',':handle_action: Deleting binding "%s" for principal %d', $_GET['bind_id'], $id );
-      if ( $session->AllowedTo('Admin')
-                || ($id > 0 && $session->principal_id == $id) ) {
+      if ($can_write_principal) {
         if ( $session->CheckConfirmationHash('GET', 'confirm') ) {
-          dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete ticket "%s" for principal %d', $_GET['bind_id'], $id );
+          dbg_error_log('admin-principal-edit',':handle_action: Allowed to delete binding "%s" for principal %d', $_GET['bind_id'], $id );
           $qry = new AwlQuery('DELETE FROM dav_binding WHERE bind_id=?;', $_GET['bind_id'] );
           if ( $qry->Exec() ) {
-            $c->messages[] = i18n('Binding deleted');
+            $c->messages[] = i18n('Binding deleted.');
             return true;
           }
           else {
@@ -151,6 +158,9 @@ function handle_subaction( $subaction ) {
           return false;
         }
       }
+      else {
+        $c->messages[] = i18n('You are not allowed to delete bindings for this principal.');
+      }
       break;
 
       default:
@@ -162,7 +172,7 @@ function handle_subaction( $subaction ) {
 function principal_editor() {
   global $c, $id, $can_write_principal, $session;
   $editor = new Editor(translate('Principal'), 'dav_principal');
-  
+
   $editor->SetLookup( 'date_format_type', "SELECT 'E', 'European' UNION SELECT 'U', 'US Format' UNION SELECT 'I', 'ISO Format'" );
   $editor->SetLookup( 'type_id', 'SELECT principal_type_id, principal_type_desc FROM principal_type ORDER BY principal_type_id' );
   $editor->SetLookup( 'locale', 'SELECT \'\', \''.translate("*** Default Locale ***").'\' UNION SELECT locale, locale_name_locale FROM supported_locales ORDER BY 1 ASC' );
@@ -170,12 +180,12 @@ function principal_editor() {
   $editor->AddAttribute( 'fullname', 'title', translate("The full name for this person, group or other type of principal.") );
   $editor->AddAttribute( 'email', 'title', translate("The email address identifies principals when processing invitations and freebusy lookups. It should be set to a unique value.") );
   $editor->SetWhere( 'principal_id='.$id );
-  
+
   $editor->AddField('is_admin', 'EXISTS( SELECT 1 FROM role_member WHERE role_no = 1 AND role_member.user_no = dav_principal.user_no )' );
   $editor->AddAttribute('is_admin', 'title', translate('An "Administrator" user has full rights to the whole DAViCal System'));
-  
+
   $post_values = false;
-  
+
   if ( isset($_POST['xxxxusername']) ) {
     $_POST['xxxxusername'] = trim(str_replace('/', '', $_POST['xxxxusername']));
     if ( $_POST['xxxxusername'] == '' ) {
@@ -190,7 +200,7 @@ function principal_editor() {
   if ( isset($_POST['email']) && trim($_POST['email']) == '' ) {
     $c->messages[] = i18n("The email address really should not be blank.");
   }
-  
+
   $pwstars = '@@@@@@@@@@';
   if ( $can_write_principal && $editor->IsSubmit() ) {
     $editor->WhereNewRecord( "principal_id=(SELECT CURRVAL('dav_id_seq'))" );
@@ -210,6 +220,9 @@ function principal_editor() {
     if ( isset($_POST['fullname']) && !isset($_POST['displayname']) ) {
       $_POST['displayname'] = $_POST['fullname'];
     }
+    if ( isset($_POST['locale']) ) {
+      awl_set_locale($_POST['locale']); // activate immediately
+    }
     if ( isset($_POST['default_privileges']) ) {
       $privilege_bitpos = array_flip($privilege_names);
       $priv_names = array_keys($_POST['default_privileges']);
@@ -228,6 +241,7 @@ function principal_editor() {
       /** We only add the default calendar if it isn't a group, and this is a create action */
       require_once('auth-functions.php');
       CreateHomeCollections($editor->Value('username'));
+      CreateDefaultRelationships($editor->Value('username'));
     }
     if ( $session->AllowedTo('Admin') ) {
       if ( $_POST['is_admin'] == 'on' ) {
@@ -287,9 +301,15 @@ function principal_editor() {
   $prompt_locale = translate('Locale');
   $prompt_type = translate('Principal Type');
   $prompt_privileges = translate('Privileges granted to All Users');
-  
-  $privs_html = build_privileges_html( $editor, 'default_privileges');
-  
+
+  if ($can_write_principal) {
+    $privs_html = build_privileges_html( $editor, 'default_privileges');
+    $submit_row = '<tr> <th class="right"></th>                   <td class="left" colspan="2">##submit##</td> </tr>';
+  } else {
+    $privs_html = principal_privilege_format_function( $editor->Value('default_privileges') );
+    $submit_row = '';
+  }
+
   $admin_row_entry = '';
   $delete_principal_button = '';
   if ( $session->AllowedTo('Admin') ) {
@@ -374,7 +394,7 @@ label.privilege {
  <tr> <th class="right">$prompt_type:</th>        <td class="left">##type_id.select##</td> </tr>
  $admin_row_entry
  <tr> <th class="right" style="white-space:normal;">$prompt_privileges:</th><td class="left">$privs_html</td> </tr>
- <tr> <th class="right"></th>                   <td class="left" colspan="2">##submit##</td> </tr>
+ $submit_row
 </table>
 </form>
 EOTEMPLATE;
@@ -386,14 +406,14 @@ EOTEMPLATE;
 
 function build_privileges_html( $ed, $fname ) {
   global $privilege_xlate, $privilege_names;
-  
+
   $btn_all = htmlspecialchars(translate('All'));             $btn_all_title = htmlspecialchars(translate('Toggle all privileges'));
   $btn_rw  = htmlspecialchars(translate('Read/Write'));      $btn_rw_title = htmlspecialchars(translate('Set read+write privileges'));
   $btn_read = htmlspecialchars(translate('Read'));           $btn_read_title = htmlspecialchars(translate('Set read privileges'));
   $btn_fb = htmlspecialchars(translate('Free/Busy'));        $btn_fb_title = htmlspecialchars(translate('Set free/busy privileges'));
   $btn_sd = htmlspecialchars(translate('Schedule Deliver')); $btn_sd_title = htmlspecialchars(translate('Privileges to allow delivery of scheduling messages'));
   $btn_ss = htmlspecialchars(translate('Schedule Send'));    $btn_ss_title = htmlspecialchars(translate('Privileges to delegate scheduling decisions'));
-  
+
   $privs_dec = bindec($ed->Value($fname));
   $privileges_set = sprintf('<div id="privileges"><input type="hidden" name="%s[fake_privilege_for_input]" value="0">%s', $fname, "\n");
   for( $i=0; $i < count($privilege_names); $i++ ) {
@@ -426,7 +446,7 @@ EOTEMPLATE;
 
 
 /**
-* privilege_format_function is for formatting the binary privileges from the
+* principal_privilege_format_function is for formatting the binary privileges from the
 * database, including localising them.  This is a hook function for a browser
 * column object, so it takes three parameters:
 * @param mixed $value The value of the column.
@@ -434,7 +454,7 @@ EOTEMPLATE;
 * @param dbrow $row The row object we read from the database.
 * @return string The formatted privileges.
 */
-function privilege_format_function( $value, $column, $row ) {
+function principal_privilege_format_function( $value, $column, $row ) {
   global $privilege_xlate;
 
   $privs = bits_to_privilege($value,'*');
@@ -448,7 +468,7 @@ function privilege_format_function( $value, $column, $row ) {
 }
 
 
-function confirm_delete_principal($confirmation_hash, $displayname ) { 
+function confirm_delete_principal($confirmation_hash, $displayname ) {
   $html = '<p class="error">';
   $html .= sprintf('<b>%s</b> \'%s\' <a class="error" href="%s&%s">%s</a> %s',
        translate('Deleting Principal:'), $displayname, $_SERVER['REQUEST_URI'],
@@ -459,7 +479,7 @@ function confirm_delete_principal($confirmation_hash, $displayname ) {
 }
 
 
-    
+
 function group_memberships_browser() {
   global $c, $id, $editor;
   $browser = new Browser(translate('Group Memberships'));
@@ -555,7 +575,7 @@ function group_members_browser() {
   $browser->AddColumn( 'members', translate('Has Members'), '', '', 'has_members_list(principal_id)' );
 
   if ( $can_write_principal ) {
-    $del_link  = '<a href="'.$c->base_url.'/admin.php?action=edit&t=principal&id='.$id.'&delete_member=##principal_id##" class="submit">'.translate('Remove').'</a>';
+    $del_link  = '<a href="'.$c->base_url.'/admin.php?action=edit&t=principal&id='.$id.'&delete_member=##principal_id##" class="submit" title="">'.translate('Remove').'</a>';
     $browser->AddColumn( 'action', translate('Action'), 'center', '', "'$edit_link&nbsp;$del_link'" );
   }
 
@@ -573,7 +593,7 @@ function group_members_browser() {
   $browser->DoQuery();
 
   if ( $can_write_principal ) {
-    $browser->RowFormat( '<tr class="r%d">', '</tr>', '#even' );
+    $browser->ExtraRowFormat( '<tr class="r%d">', '</tr>', '#even' );
     $extra_row = array( 'group_id' => -1 );
     $browser->MatchedRow('group_id', -1, 'edit_group_row');
     $extra_row = (object) $extra_row;
@@ -624,9 +644,9 @@ function grant_row_editor() {
 }
 
 
-function edit_grant_row( $row_data ) {
+function edit_grant_row_principal( $row_data ) {
   global $id, $grantrow;
-  
+
   $orig_to_id = intval($row_data->to_principal);
   if ( $orig_to_id > -1 ) {
     $grantrow->SetRecord( $row_data );
@@ -634,7 +654,7 @@ function edit_grant_row( $row_data ) {
   else {
     $grantrow->Initialise( $row_data );
   }
-  
+
   $privs_html = build_privileges_html( $grantrow, 'grant_privileges' );
 
   $form_id = $grantrow->Id();
@@ -655,7 +675,7 @@ EOTEMPLATE;
   return $grantrow->Render();
 }
 
-    
+
 function principal_grants_browser() {
   global $c, $id, $editor, $can_write_principal;
     $browser = new Browser(translate('Principal Grants'));
@@ -665,12 +685,12 @@ function principal_grants_browser() {
   $browser->AddHidden( 'principal_link', "'<a href=\"$rowurl' || to_principal || '\">' || to_principal || '</a>'" );
   $browser->AddHidden( 'grant_privileges', 'privileges' );
   $browser->AddColumn( 'displayname', translate('Display Name') );
-  $browser->AddColumn( 'privs', translate('Privileges'), '', '', 'privileges', '', '', 'privilege_format_function' );
+  $browser->AddColumn( 'privs', translate('Privileges'), '', '', 'privileges', '', '', 'principal_privilege_format_function' );
   $browser->AddColumn( 'members', translate('Has Members'), '', '', 'has_members_list(principal_id)' );
 
   if ( $can_write_principal ) {
-    $del_link  = '<a href="'.$c->base_url.'/admin.php?action=edit&t=principal&id='.$id.'&delete_grant=##to_principal##" class="submit">'.translate('Revoke').'</a>';
-    $edit_link  = '<a href="'.$c->base_url.'/admin.php?action=edit&t=principal&id='.$id.'&edit_grant=##to_principal##" class="submit">'.translate('Edit').'</a>';
+    $del_link  = '<a href="'.$c->base_url.'/admin.php?action=edit&t=principal&id='.$id.'&delete_grant=##to_principal##" class="submit" title="">'.translate('Revoke').'</a>';
+    $edit_link  = '<a href="'.$c->base_url.'/admin.php?action=edit&t=principal&id='.$id.'&edit_grant=##to_principal##" class="submit" title="">'.translate('Edit').'</a>';
     $browser->AddColumn( 'action', translate('Action'), 'center', '', "'$edit_link&nbsp;$del_link'" );
   }
 
@@ -690,12 +710,12 @@ function principal_grants_browser() {
 
   if ( $can_write_principal ) {
     if ( isset($_GET['edit_grant']) ) {
-      $browser->MatchedRow('to_principal', $_GET['edit_grant'], 'edit_grant_row');
+      $browser->MatchedRow('to_principal', $_GET['edit_grant'], 'edit_grant_row_principal');
     }
     else if ( isset($id ) ) {
-      $browser->RowFormat( '<tr class="r%d">', '</tr>', '#even' );
+      $browser->ExtraRowFormat( '<tr class="r%d">', '</tr>', '#even' );
       $extra_row = array( 'to_principal' => -1 );
-      $browser->MatchedRow('to_principal', -1, 'edit_grant_row');
+      $browser->MatchedRow('to_principal', -1, 'edit_grant_row_principal');
       $extra_row = (object) $extra_row;
       $browser->AddRow($extra_row);
     }
@@ -710,15 +730,15 @@ function ticket_row_editor() {
   $ticketrow = new Editor("Tickets", "access_ticket");
   $ticketrow->SetSubmitName( 'ticketrow' );
   if ( $can_write_principal && $ticketrow->IsSubmit() ) {
-    
+
     $username = $editor->Value('username');
     $ugly_path = $_POST['target'];
     if ( $ugly_path == '/'.$username || $ugly_path == '/'.$username.'/' ) {
       $target_collection = $id;
     }
-    else { 
+    else {
       $username_len = strlen($username) + 2;
-      $sql = "SELECT collection_id FROM collection WHERE dav_name = :exact_name"; 
+      $sql = "SELECT collection_id FROM collection WHERE dav_name = :exact_name";
       $sql .= " AND substring(dav_name FROM 1 FOR $username_len) = '/$username/'";
       $params = array( ':exact_name' => $ugly_path );
       if ( !preg_match( '#/$#', $ugly_path ) ) {
@@ -737,7 +757,7 @@ function ticket_row_editor() {
         return $ticketrow;
       }
     }
-    
+
     $_POST['dav_owner_id'] = $id;
     $_POST['target_collection_id'] = $target_collection;
     $ticket_id = check_by_regex($_POST['ticket_id'], '/[A-Za-z0-9]+/');
@@ -792,12 +812,16 @@ function access_ticket_browser() {
 
   $browser = new Browser(translate('Access Tickets'));
 
-  $browser->AddColumn( 'ticket_id', translate('Ticket ID'), '', '' );
+  if ( $can_write_principal ) {
+    $browser->AddColumn( 'ticket_id', translate('Ticket ID'), '', '' );
+  }
   $browser->AddColumn( 'target', translate('Target'), '', '<td style="white-space:nowrap;">%s</td>', "COALESCE(d.dav_name,c.dav_name)" );
   $browser->AddColumn( 'expires', translate('Expires'), '', '', 'TO_CHAR(expires,\'YYYY-MM-DD HH:MI:SS\')');
-  $browser->AddColumn( 'privs', translate('Privileges'), '', '', 'privileges', '', '', 'privilege_format_function' );
-  $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id='.$id.'&ticket_id=##URL:ticket_id##&subaction=delete_ticket';
-  $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+  $browser->AddColumn( 'privs', translate('Privileges'), '', '', 'privileges', '', '', 'principal_privilege_format_function' );
+  if ($can_write_principal) {
+    $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id='.$id.'&ticket_id=##URL:ticket_id##&subaction=delete_ticket';
+    $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+  }
 
   $browser->SetOrdering( 'target', 'A' );
 
@@ -820,7 +844,7 @@ function access_ticket_browser() {
   return $browser;
 }
 
-  
+
 function confirm_delete_ticket($confirmation_hash) {
   $html = '<table><tr><td class="error">';
   $html .= sprintf('<b>%s</b> "%s" <a class="error" href="%s&%s">%s</a> %s',
@@ -834,7 +858,7 @@ function confirm_delete_ticket($confirmation_hash) {
 
 
 function principal_collection_browser() {
-  global $c, $page_elements, $id, $editor;
+  global $c, $page_elements, $id, $editor, $can_write_principal;
 
   $browser = new Browser(translate('Principal Collections'));
 
@@ -846,18 +870,23 @@ function principal_collection_browser() {
   $browser->AddColumn( 'publicly_readable', translate('Public'), 'centre', '', 'CASE WHEN publicly_readable THEN \''.translate('Yes').'\' ELSE \''.translate('No').'\' END' );
   $browser->AddColumn( 'privs', translate('Privileges'), '', '',
           "COALESCE( privileges_list(default_privileges), '[".translate('from principal')."]')" );
-  $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id='.$id.'&dav_name=##URL:dav_name##&subaction=delete_collection';
-  $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+  if ($can_write_principal) {
+    $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id='.$id.'&dav_name=##URL:dav_name##&subaction=delete_collection';
+    $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\" title=\"\">".translate('Delete')."</a>'" );
+  }
 
   $browser->SetOrdering( 'dav_name', 'A' );
 
   $browser->SetJoins( "collection " );
   $browser->SetWhere( 'user_no = '.intval($editor->Value('user_no')) );
 
-  $browser->AddRow( array( 'dav_name' => '<a href="'.$rowurl.'&user_no='.intval($editor->Value('user_no')).'" class="submit">'.translate('Create Collection').'</a>' ));
+  if ($can_write_principal) {
+    $browser->ExtraRowFormat( '<tr class="r%d">', '</tr>', '#even' );
+    $browser->AddRow( array( 'dav_name' => '<a href="'.$rowurl.'&user_no='.intval($editor->Value('user_no')).'" class="submit">'.translate('Create Collection').'</a>' ));
+  }
 
   if ( $c->enable_row_linking ) {
-    $browser->RowFormat( '<tr onMouseover="LinkHref(this,1);" title="'.translate('Click to edit principal details').'" class="r%d">', '</tr>', '#even' );
+    $browser->RowFormat( '<tr onMouseover="LinkHref(this,1);" title="'.translate('Click to edit collection details').'" class="r%d">', '</tr>', '#even' );
   }
   else {
     $browser->RowFormat( '<tr class="r%d">', '</tr>', '#even' );
@@ -865,7 +894,7 @@ function principal_collection_browser() {
   $browser->DoQuery();
   return $browser;
 }
-  
+
 function confirm_delete_collection($confirmation_hash) {
   $html = '<table><tr><td class="error">';
   $html .= sprintf('<b>%s</b> "%s" <a class="error" href="%s&%s">%s</a> %s',
@@ -877,20 +906,147 @@ function confirm_delete_collection($confirmation_hash) {
   return $html;
 }
 
+
+function binding_row_editor() {
+  global $c, $id, $editor, $can_write_principal;
+
+  $bindingrow = new Editor("Bindings", "dav_binding");
+  $bindingrow->SetSubmitName( 'bindingrow' );
+  if ( $can_write_principal && $bindingrow->IsSubmit() ) {
+    if ( substr($_POST['dav_name'], -1) != '/' ) {
+      $_POST['dav_name'] .= '/';
+    }
+
+    $dav_name = $_POST['dav_name'];
+    $parent = '/'.$editor->Value('username').'/';
+    if ( strpos($dav_name, $parent) !== 0 ) {
+      $c->messages[] = translate("Can only bind collections into the current principal's namespace");
+      return $bindingrow;
+    }
+    if ( substr_count($dav_name, '/') != 3 || substr_count($dav_name, '\\') > 0 ) {
+      $c->messages[] = translate("Bound As is invalid");
+      return $bindingrow;
+    }
+    $qry = new AwlQuery('SELECT dav_name FROM collection where dav_name = :dav_name UNION SELECT dav_name FROM dav_binding WHERE dav_name = :dav_name', array( ':dav_name' => $dav_name) );
+    if ( $qry->Exec('dav_name') && $qry->rows() > 0 ) {
+      $c->messages[] = translate('A resource already exists at the destination.');
+      return $bindingrow;
+    }
+
+    if ( empty($_POST['access_ticket_id']) )
+      $_POST['access_ticket_id'] = null;
+    $_POST['dav_owner_id'] = $id;
+    $_POST['parent_container'] = $parent;
+
+    //  external binds shouldn't ever point back to ourselves but they should be a valid http[s] url
+    $href = $_POST['source'];
+    if ( preg_match ( '{^(?:https?://|file:///)([^/]+)(:[0-9]\+)?/.+$}', $href, $matches )
+        && strcasecmp( $matches[0], 'localhost' ) !== 0 && strcasecmp( $matches[0], '127.0.0.1' ) !== 0
+        && strcasecmp( $matches[0], $_SERVER['SERVER_NAME'] ) !== 0 && strcasecmp( $matches[0], $_SERVER['SERVER_ADDR'] ) !== 0
+    ) {
+      $path = '/.external/' . md5($href);
+      $qry->QDo('SELECT collection_id FROM collection WHERE dav_name = :dav_name ', array( ':dav_name' => $path ));
+      if ( $qry->rows() == 1 && ($row = $qry->Fetch()) ) {
+        $dav_id = $row->collection_id;
+      }
+      else {
+        $qry->QDo( 'INSERT INTO collection ( user_no, parent_container, dav_name, dav_etag, dav_displayname,
+                                              is_calendar, is_addressbook, resourcetypes, created )
+                                VALUES( :user_no, :parent_container, :dav_name, :dav_etag, :dav_displayname,
+                                        :is_calendar, :is_addressbook, :resourcetypes, current_timestamp )',
+            array(
+              ':user_no'          => $editor->Value('user_no'),
+              ':parent_container' => '/.external/',
+              ':dav_name'         => $path,
+              ':dav_etag'         => md5( $editor->Value('user_no') . $path ),
+              ':dav_displayname'  => $_POST['dav_displayname'],
+              ':is_calendar'      => 't',
+              ':is_addressbook'   => 'f',
+              ':resourcetypes'    => '<DAV::collection/><urn:ietf:params:xml:ns:caldav:calendar/>'
+            )
+        );
+
+        $qry->QDo('SELECT collection_id FROM collection WHERE dav_name = :dav_name ', array( ':dav_name' => $path ));
+        if ( $qry->rows() != 1 || !($row = $qry->Fetch()) ) {
+          $c->messages[] = translate('Database Error');
+          return $bindingrow;
+        }
+        $dav_id = $row->collection_id;
+      }
+
+      $_POST['bound_source_id'] = $dav_id;
+      $_POST['external_url'] = $href;
+      $_POST['external_type'] = 'calendar';
+    }
+    else {
+      // internal bind
+      require_once('DAVResource.php');
+      $source = new DAVResource( $href );
+      if ( !$source->Exists() || $source->IsPrincipal() || !$source->IsCollection() || $source->dav_name() == '/' ) {
+        $c->messages[] = translate('The BIND Request MUST identify an existing resource.');
+        return $bindingrow;
+      }
+      if ( $source->IsBinding() )
+        $source = new DAVResource( $source->bound_from() );
+      $_POST['bound_source_id'] = $source->collection_id();
+    }
+
+    $c->messages[] = 'Creating new binding for this principal';
+    $bindingrow->SetWhere( "dav_name = '$dav_name'" );
+    $bindingrow->Write();
+  }
+  return $bindingrow;
+}
+
+
+function edit_binding_row( $row_data ) {
+  global $id, $bindingrow;
+
+  if ( isset($row_data->dav_name) ) {
+    $bindingrow->Initialise( $row_data );
+  }
+
+  $form_id = $bindingrow->Id();
+  $form_url = preg_replace( '#&(edit|delete)_[a-z]+=\d+#', '', $_SERVER['REQUEST_URI'] );
+  $source_title = translate('Path to collection you wish to bind, like /user1/calendar/ or https://cal.example.com/user2/cal/');
+  $access_title = translate('optional');
+
+  $template = <<<EOTEMPLATE
+<form method="POST" enctype="multipart/form-data" id="form_$form_id" action="$form_url">
+  <td class="left">&nbsp;<input type="hidden" name="id" value="$id"></td>
+  <td class="left"><input type="text" name="dav_name" value="$row_data->dav_name" size="25"></td>
+  <td class="left"><input type="text" name="dav_displayname" size="20"></td>
+  <td class="left"><input type="text" name="source" size="40" title="$source_title"></td>
+  <td class="left"><input type="text" name="access_ticket_id" size="10" title="$access_title"></td>
+  <td class="left">&nbsp;</td>
+  <td class="center">##submit##</td>
+</form>
+EOTEMPLATE;
+
+  $bindingrow->SetTemplate( $template );
+  $bindingrow->Title("");
+
+  return $bindingrow->Render();
+}
+
+
 function bindings_to_other_browser() {
-  global $c, $page_elements, $id, $editor;
+  global $c, $editor, $can_write_principal;
   $browser = new Browser(translate('Bindings to other collections'));
   $browser->AddColumn( 'bind_id', translate('ID'), '', '' );
   $browser->AddHidden( 'b.dav_owner_id' );
   $browser->AddHidden( 'p.principal_id' );
   $browser->AddColumn( 'bound_as', translate('Bound As'), '', '<td style="white-space:nowrap;">%s</td>', 'b.dav_name' );
+  $browser->AddColumn( 'dav_displayname', translate('Display Name'), '', '', 'b.dav_displayname' );
   $browser->AddColumn( 'dav_name', translate('To Collection'), '', '<td style="white-space:nowrap;">%s</td>', 'c.dav_name' );
   $browser->AddColumn( 'access_ticket_id', translate('Ticket ID'), '', '' );
   $browser->AddColumn( 'privs', translate('Privileges'), '', '', "privileges_list(privileges)" );
-  $delurl = $c->base_url . sprintf('/admin.php?action=edit&t=principal&id=%s&bind_id=##bind_id##&subaction=delete_bind_in', $editor->Value('principal_id'));
-  $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+  if ($can_write_principal) {
+    $delurl = $c->base_url . sprintf('/admin.php?action=edit&t=principal&id=%s&bind_id=##bind_id##&subaction=delete_bind_in', $editor->Value('principal_id'));
+    $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+  }
 
-  $browser->SetOrdering( 'target', 'A' );
+  $browser->SetOrdering( 'bound_as', 'A' );
 
   $browser->SetJoins( 'dav_binding b LEFT JOIN collection c ON (bound_source_id=collection_id) LEFT JOIN access_ticket t ON (ticket_id=access_ticket_id) LEFT JOIN principal p USING(user_no)' );
   $browser->SetWhere( 'b.dav_name ~ '.sprintf("'^/%s/'", $editor->Value('username')) );
@@ -898,9 +1054,17 @@ function bindings_to_other_browser() {
   $browser->RowFormat( '<tr class="r%d">', '</tr>', '#even' );
 
   $browser->DoQuery();
+
+  if ( $can_write_principal ) {
+    $extra_row = (object) array( 'bind_id' => -1,
+                                 'dav_name' => '/'.$editor->Value('username').'/boundcalendar/'
+                               );
+    $browser->MatchedRow('bind_id', -1, 'edit_binding_row');
+    $browser->AddRow($extra_row);
+  }
   return $browser;
 }
-  
+
 function confirm_delete_bind_in($confirmation_hash) {
   $html = '<table><tr><td class="error">';
   $html .= sprintf('<b>%s</b> "%s" <a class="error" href="%s&%s">%s</a> %s',
@@ -911,10 +1075,10 @@ function confirm_delete_bind_in($confirmation_hash) {
   $html .= "</td></tr></table>\n";
   return $html;
 }
-  
-  
+
+
 function bindings_to_us_browser() {
-  global $c, $page_elements, $id, $editor;
+  global $c, $editor, $session;
   $browser = new Browser(translate('Bindings to this Principal\'s Collections'));
   $browser->AddColumn( 'bind_id', translate('ID'), '', '' );
   $browser->AddHidden( 'b.dav_owner_id' );
@@ -923,10 +1087,12 @@ function bindings_to_us_browser() {
   $browser->AddColumn( 'bound_as', translate('Bound As'), '', '<td style="white-space:nowrap;">%s</td>', 'b.dav_name' );
   $browser->AddColumn( 'access_ticket_id', translate('Ticket ID'), '', '' );
   $browser->AddColumn( 'privs', translate('Privileges'), '', '', "privileges_list(privileges)" );
-  $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id=##principal_id##&bind_id=##bind_id##&subaction=delete_binding';
-  $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+  if ( $session->AllowedTo('Admin') ) {
+    $delurl = $c->base_url . '/admin.php?action=edit&t=principal&id=##principal_id##&bind_id=##bind_id##&subaction=delete_binding';
+    $browser->AddColumn( 'delete', translate('Action'), 'center', '', "'<a class=\"submit\" href=\"$delurl\">".translate('Delete')."</a>'" );
+  }
 
-  $browser->SetOrdering( 'target', 'A' );
+  $browser->SetOrdering( 'dav_name', 'A' );
 
   $browser->SetJoins( 'dav_binding b LEFT JOIN collection c ON (bound_source_id=collection_id) LEFT JOIN access_ticket t ON (ticket_id=access_ticket_id) LEFT JOIN principal p USING(user_no)' );
   $browser->SetWhere( 'p.principal_id = '.intval($editor->Value('principal_id')) );
@@ -936,7 +1102,7 @@ function bindings_to_us_browser() {
   $browser->DoQuery();
   return $browser;
 }
-  
+
 function confirm_delete_binding( $confirmation_hash ) {
   $html = '<table><tr><td class="error">';
   $html .= sprintf('<b>%s</b> "%s" <a class="error" href="%s&%s">%s</a> %s',
@@ -951,7 +1117,7 @@ function confirm_delete_binding( $confirmation_hash ) {
 
 if ( isset($_GET['subaction']) ) {
   if ( handle_subaction($_GET['subaction']) && 'delete_principal' == $_GET['subaction'] ) {
-    return true;    
+    return true;
   }
 }
 
@@ -964,8 +1130,8 @@ if ( isset($id) && $id > 0 ) {
 
   if ( isset($delete_principal_confirmation_required) )
     $page_elements[] = confirm_delete_principal($delete_principal_confirmation_required, $editor->Value('displayname'));
-  
-  
+
+
   $page_elements[] = group_memberships_browser();
   if ( $editor->Value('type_id') == 3 ) {
     $grouprow = group_row_editor();
@@ -973,18 +1139,19 @@ if ( isset($id) && $id > 0 ) {
   }
   $grantrow = grant_row_editor();
   $page_elements[] = principal_grants_browser();
-  if ( isset($delete_grant_confirmation_required) ) $page_elements[] = confirm_delete_grant($delete_grant_confirmation_required);  
-  
+  if ( isset($delete_grant_confirmation_required) ) $page_elements[] = confirm_delete_grant($delete_grant_confirmation_required);
+
   $ticketrow = ticket_row_editor();
   $page_elements[] = access_ticket_browser();
-  if ( isset($delete_ticket_confirmation_required) ) $page_elements[] = confirm_delete_ticket($delete_ticket_confirmation_required);  
+  if ( isset($delete_ticket_confirmation_required) ) $page_elements[] = confirm_delete_ticket($delete_ticket_confirmation_required);
 
   $page_elements[] = principal_collection_browser();
-  if ( isset($delete_collection_confirmation_required) ) $page_elements[] = confirm_delete_collection($delete_collection_confirmation_required);  
-  
+  if ( isset($delete_collection_confirmation_required) ) $page_elements[] = confirm_delete_collection($delete_collection_confirmation_required);
+
+  $bindingrow = binding_row_editor();
   $page_elements[] = bindings_to_other_browser();
-  if ( isset($delete_bind_in_confirmation_required) ) $page_elements[] = confirm_delete_bind_in($delete_bind_in_confirmation_required);  
-  
+  if ( isset($delete_bind_in_confirmation_required) ) $page_elements[] = confirm_delete_bind_in($delete_bind_in_confirmation_required);
+
   $page_elements[] = bindings_to_us_browser();
   if ( isset($delete_binding_confirmation_required) ) $page_elements[] = confirm_delete_binding($delete_binding_confirmation_required);
 }
